﻿using BankContracts.BindingModels;
using BankContracts.BusinessLogicsContracts;
using BankContracts.SearchModels;
using BankContracts.StoragesContracts;
using BankContracts.ViewModels;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BankBusinessLogic.BusinessLogics
{
    public class TransferLogic : ITransferLogic
    {
        private readonly ILogger _logger;
        private readonly ITransferStorage _transferStorage;

        public TransferLogic(ILogger<TransferLogic> logger, ITransferStorage transferStorage)
        {
            _logger = logger;
            _transferStorage = transferStorage;
        }

        public List<TransferViewModel>? ReadList(TransferSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id:{Id}", model?.Id);
            var list = model == null ? _transferStorage.GetFullList() : _transferStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count:{Count}", list.Count);
            return list;
        }

        public TransferViewModel? ReadElement(TransferSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id:{Id}", model.Id);
            var element = _transferStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id:{Id}", element.Id);
            return element;
        }

        public bool Create(TransferBindingModel model)
        {
            CheckModel(model);
            if (_transferStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(TransferBindingModel model)
        {
            CheckModel(model);
            if (_transferStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(TransferBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_transferStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(TransferBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (model.Amount <= 0)
            {
                throw new InvalidOperationException("Сумма зачисления не может быть меньше 0");
            }
            _logger.LogInformation("Transfer. TransferDateTime:{TransferDateTime}. Amount:{Amount}. PaymentId:{PaymentId}. Id:{Id}", model.TransferDateTime, model.Amount, model.PaymentId, model.Id);
        }
    }
}
